//Copyright 2005 Nokia Corporation.
//
// THIS SOURCE CODE IS PROVIDED 'AS IS', WITH NO WARRANTIES WHATSOEVER,
// EXPRESS OR IMPLIED, INCLUDING ANY WARRANTY OF MERCHANTABILITY, FITNESS
// FOR ANY PARTICULAR PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE
// OR TRADE PRACTICE, RELATING TO THE SOURCE CODE OR ANY WARRANTY OTHERWISE
// ARISING OUT OF ANY PROPOSAL, SPECIFICATION, OR SAMPLE AND WITH NO
// OBLIGATION OF NOKIA TO PROVIDE THE LICENSEE WITH ANY MAINTENANCE OR
// SUPPORT. FURTHERMORE, NOKIA MAKES NO WARRANTY THAT EXERCISE OF THE
// RIGHTS GRANTED HEREUNDER DOES NOT INFRINGE OR MAY NOT CAUSE INFRINGEMENT
// OF ANY PATENT OR OTHER INTELLECTUAL PROPERTY RIGHTS OWNED OR CONTROLLED
// BY THIRD PARTIES
//
// Furthermore, information provided in this source code is preliminary,
// and may be changed substantially prior to final release. Nokia Corporation
// retains the right to make changes to this source code at
// any time, without notice. This source code is provided for informational
// purposes only.
//
// Nokia and Nokia Connecting People are registered trademarks of Nokia
// Corporation.
// Java and all Java-based marks are trademarks or registered trademarks of
// Sun Microsystems, Inc.
// Other product and company names mentioned herein may be trademarks or
// trade names of their respective owners.
//
// A non-exclusive, non-transferable, worldwide, limited license is hereby
// granted to the Licensee to download, print, reproduce and modify the
// source code. The licensee has the right to market, sell, distribute and
// make available the source code in original or modified form only when
// incorporated into the programs developed by the Licensee. No other
// license, express or implied, by estoppel or otherwise, to any other
// intellectual property rights is granted herein.

package mmsmidlet;

import javax.microedition.lcdui.*;
import javax.microedition.media.*;
import javax.microedition.media.control.*;
import java.io.IOException;

// The CameraScreen class shows the live view
// of the camera using the MMAPI and gives
// commands to capture the contents of the camera
class CameraScreen
  extends Canvas
  implements CommandListener {
  private final MMSMIDlet midlet;
  private final Command exitCommand;
  private Player player = null;
  private Command captureCommand = null;
  private VideoControl videoControl = null;
  private boolean active = false;

  CameraScreen(MMSMIDlet midlet) {
    this.midlet = midlet;
  
    // Builds the user interface
    exitCommand = new Command("Exit", Command.EXIT, 1);
  	addCommand(exitCommand);
  	captureCommand = new Command("Capture", Command.SCREEN, 1);
  	addCommand(captureCommand);
  	setCommandListener(this);		
  }
  
  // Paint the canvas' background in black
  public void paint(Graphics g) {
  	// black background
  	g.setColor(0x00000000); 
  	g.fillRect(0, 0, getWidth(), getHeight());
  }
  
  public void commandAction(Command c, Displayable d) {
    if (c == exitCommand) {
      midlet.exitApplication();
    } else if (c == captureCommand) {
      captureImage();
    }
  }
  
  public void keyPressed(int keyCode) {
    if (getGameAction(keyCode) == FIRE) {
      captureImage();
    }
  }
  
  // Creates and starts the video player
  synchronized void start() {
    try {
  	  player = Manager.createPlayer("capture://video");
  	  player.realize();
  
  	  // Get VideoControl for the viewfinder
  	  videoControl = (VideoControl)player.getControl("VideoControl");
  	  if (videoControl == null) {
  	    discardPlayer();
      	midlet.showError("Cannot get the video control.\n"
      	    +"Capture may not be supported.");
      	player = null;
  	  } else {
  	    // Set up the viewfinder on the screen.
  	    videoControl.initDisplayMode(VideoControl.USE_DIRECT_VIDEO,
	          this);
      	int canvasWidth = getWidth();
      	int canvasHeight = getHeight();
      	int displayWidth = videoControl.getDisplayWidth();
      	int displayHeight = videoControl.getDisplayHeight();
      	int x = (canvasWidth - displayWidth) / 2;
      	int y = (canvasHeight - displayHeight) / 2;
      	videoControl.setDisplayLocation(x, y);
      	player.start();
      	videoControl.setVisible(true);
  	  }
    } catch (IOException ioe) {
      discardPlayer();
      midlet.showError("IOException: " + ioe.getMessage());
    } catch (MediaException me) {
      midlet.showError("MediaException: " + me.getMessage());
    } catch (SecurityException se) {
      midlet.showError("SecurityException: " + se.getMessage());
    }
  }
  
  // Stops the video player
  synchronized void stop() {
    if (player != null) {
      try {
        videoControl.setVisible(false);
        player.stop();
      } catch (MediaException me) {
        midlet.showError("MediaException: " + me.getMessage());
      }
      active = false;
    }
  }
  
  // this method will discard the video player
  private void discardPlayer() {
    if (player != null) {
      player.deallocate();
      player.close();
      player = null;
    }
    videoControl = null;
  }
  
  // captures the image from the video player
  // in a separate thread
  private void captureImage() {
    if (player != null) {
      // Capture image in a new thread.
      new Thread() {
        public void run() {
          try {
            byte[] jpgImage =
              videoControl.getSnapshot("encoding=jpeg");
            midlet.imageCaptured(jpgImage);
            discardPlayer();
          } catch (MediaException me) {
            midlet.showError("MediaException: " + me.getMessage());
          } catch (SecurityException se) {
            midlet.showError("SecurityException: " + se.getMessage());
          }
        }
      }.start();
    }
  }
}
